from typing import Optional, List
from molten import schema, Field
from molten.validation.field import StringValidator
from sqlalchemy import Column, Text, Boolean, ForeignKey, BigInteger
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm import relationship
from databag.db import Base, DBMixin
from databag.schema import Link

# TODO Add object choice
TYPE_CHOICES = [""]


# @schema
# class Property:
#     name: str
#     type: str = Field(validator=StringValidator, choices=TYPE_CHOICES)
#     required: bool = Field(default=True)
#     validator:


# @schema
# class Bag:
#     id: int = Field(response_only=True)
#     created_date: str = Field(response_only=True)
#     modified_date: str = Field(response_only=True)
#     name: str
#     description: Optional[str]
#     schema: str
#     is_active: Optional[bool] = Field(default=True)
#     href: Link = Field(response_only=True)


class BagSchemaModel(Base, DBMixin):
    __tablename__ = "schema"
    bag_id = Column(BigInteger, ForeignKey("bag.id"), nullable=True)
    schema = Column(JSONB, nullable=False)
    bag = relationship("BagModel", back_populates="schemas")


class BagModel(Base, DBMixin):
    __tablename__ = "bag"
    name = Column(Text)
    description = Column(Text, nullable=True)
    current_schema_id = Column(BigInteger, ForeignKey("schema.id"), nullable=True)
    is_active = Column(Boolean, default=True)
    schemas = relationship(
        "BagSchemaModel", order_by="BagSchemaModel.id", back_populates="bag"
    )


class BagEntryModel(Base, DBMixin):
    __tablename__ = "entry"
    bag_id = Column(BigInteger, ForeignKey("bag.id"))
    write_schema_id = Column(BigInteger, ForeignKey("schema.id"))
    data = Column(JSONB, nullable=False)
